@@ -1,4 +1,4 @@
-# Copyright (c) 1998-2002 by Jonathan Swartz. All rights reserved.
+# Copyright (c) 1998-2005 by Jonathan Swartz. All rights reserved.
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 
@@ -7,58 +7,69 @@ package HTML::Mason::Component;
 use strict;
 use File::Spec;
 use HTML::Mason::Exceptions( abbr => [qw(param_error)] );
-use HTML::Mason::Tools qw(absolute_comp_path);
+use HTML::Mason::Tools qw(absolute_comp_path can_weaken);
 use Params::Validate qw(:all);
 Params::Validate::validation_options( on_fail => sub { param_error join '', @_  } );
 
-use HTML::Mason::Container;
-use base qw(HTML::Mason::Container);
-
 use HTML::Mason::Exceptions( abbr => ['error'] );
 use HTML::Mason::MethodMaker
     ( read_only => [ qw( code
-			 load_time
-			 declared_args
 			 comp_id
+			 compiler_id
+			 declared_args
 			 inherit_path
 			 inherit_start_path
-                         interp
-			 object_size
-			 compiler_id ) ],
+                         has_filter
+			 load_time
+                       ) ],
 
       read_write => [ [ dynamic_subs_request => { isa => 'HTML::Mason::Request' } ],
 		      [ mfu_count => { type => SCALAR } ],
+                      [ filter => { type => CODEREF } ],
 		    ]
       );
 
-__PACKAGE__->valid_params
-    (
-     attr               => {type => HASHREF, default => {}, public => 0},
-     code               => {type => CODEREF, public => 0, public => 0},
-     load_time          => {type => SCALAR,  optional => 1, public => 0},
-     declared_args      => {type => HASHREF, default => {}, public => 0},
-     dynamic_subs_init  => {type => CODEREF, default => sub {}, public => 0},
-     flags              => {type => HASHREF, default => {}, public => 0},
-     comp_id            => {type => SCALAR,  optional => 1, public => 0},
-     methods            => {type => HASHREF, default => {}, public => 0},
-     mfu_count          => {type => SCALAR,  default => 0, public => 0},
-     object_size        => {type => SCALAR,  default => 0, public => 0},
-     parser_version     => {type => SCALAR,  optional => 1, public => 0}, # allows older components to be instantied
-     compiler_id        => {type => SCALAR,  optional => 1, public => 0},
-     subcomps           => {type => HASHREF, default => {}, public => 0},
-    );
+# for reference later
+# 
+# __PACKAGE__->valid_params
+#     (
+#      attr               => {type => HASHREF, default => {}, public => 0},
+#      code               => {type => CODEREF, public => 0, public => 0},
+#      load_time          => {type => SCALAR,  optional => 1, public => 0},
+#      declared_args      => {type => HASHREF, default => {}, public => 0},
+#      dynamic_subs_init  => {type => CODEREF, default => sub {}, public => 0},
+#      flags              => {type => HASHREF, default => {}, public => 0},
+#      comp_id            => {type => SCALAR,  optional => 1, public => 0},
+#      methods            => {type => HASHREF, default => {}, public => 0},
+#      mfu_count          => {type => SCALAR,  default => 0, public => 0},
+#      parser_version     => {type => SCALAR,  optional => 1, public => 0}, # allows older components to be instantied
+#      compiler_id        => {type => SCALAR,  optional => 1, public => 0},
+#      subcomps           => {type => HASHREF, default => {}, public => 0},
+#     );
+# 
 
+my %defaults = ( attr              => {},
+                 declared_args     => {},
+                 dynamic_subs_init => sub {},
+                 flags             => {},
+                 methods           => {},
+                 mfu_count         => 0,
+                 subcomps          => {},
+               );
 sub new
 {
     my $class = shift;
-    my $self = $class->SUPER::new(@_);
+    my $self = bless { %defaults, @_ }, $class;
 
-    # Initialize subcomponent and method properties.
+    # Initialize subcomponent and method properties: owner, name, and
+    # is_method flag.
     while (my ($name,$c) = each(%{$self->{subcomps}})) {
-	$c->assign_subcomponent_properties($self,$name);
+	$c->assign_subcomponent_properties($self,$name,0);
+	Scalar::Util::weaken($c->{owner}) if can_weaken;
     }
     while (my ($name,$c) = each(%{$self->{methods}})) {
-	$c->assign_subcomponent_properties($self,$name);
+	$c->assign_subcomponent_properties($self,$name,1);
+	Scalar::Util::weaken($c->{owner}) if can_weaken;
     }
 
     return $self;
@@ -66,19 +77,33 @@ sub new
 
 my $comp_count = 0;
 sub assign_runtime_properties {
-    my ($self, $interp, $info) = @_;
-    $self->{interp} = $interp;
-    $self->{comp_id} = defined $info->comp_id ? $info->comp_id : "[anon ". ++$comp_count . "]";
+    my ($self, $interp, $source) = @_;
+    $self->interp($interp);
+    $self->{comp_id} = defined $source->comp_id ? $source->comp_id : "[anon ". ++$comp_count . "]";
 
-    $self->{path} = $info->comp_path;
+    $self->{path} = $source->comp_path;
 
     $self->_determine_inheritance;
 
-    foreach my $c (values(%{$self->{subcomps}})) {
-	$c->assign_runtime_properties($interp, $info);
+    foreach my $c (values(%{$self->{subcomps}}), values(%{$self->{methods}})) {
+	$c->assign_runtime_properties($interp, $source);
+    }
+
+    # Cache of uncanonicalized call paths appearing in the
+    # component. Used in $m->fetch_comp.
+    #
+    if ($interp->use_internal_component_caches) {
+	$self->{fetch_comp_cache} = {};
     }
 }
 
+sub flush_internal_caches
+{
+    my ($self) = @_;
+
+    $self->{fetch_comp_cache} = {};
+}
+ 
 sub _determine_inheritance {
     my $self = shift;
 
@@ -89,7 +114,7 @@ sub _determine_inheritance {
 	if (defined($self->{flags}->{inherit})) {
 	    $self->{inherit_path} = absolute_comp_path($self->{flags}->{inherit}, $self->dir_path);
 	}
-    } else {
+    } elsif ( $interp->use_autohandlers ) {
 	if ($self->name eq $interp->autohandler_name) {
 	    unless ($self->dir_path eq '/') {
 		($self->{inherit_start_path}) = $self->dir_path =~ m,^(.*/)?.*,s
@@ -105,13 +130,17 @@ sub run {
 
     $self->{mfu_count}++;
 
-    # Note: this must always preserve calling wantarray() context
-    return $self->{code}->(@_);
+    $self->{code}->(@_);
 }
 
 sub dynamic_subs_init {
     my $self = shift;
 
+    error "cannot call a method or subcomponent from a <%shared> block"
+        if $self->{in_dynamic_subs_init};
+
+    local $self->{in_dynamic_subs_init} = 1;
+
     $self->{dynamic_subs_hash} = $self->{dynamic_subs_init}->();
     error "could not process <%shared> section (does it contain a return()?)"
 	unless ref($self->{dynamic_subs_hash}) eq 'HASH';
@@ -123,7 +152,7 @@ sub run_dynamic_sub {
     error "call_dynamic: assert error - could not find code for key $key in component " . $self->title
 	unless exists $self->{dynamic_subs_hash}->{$key};
 
-    $self->{dynamic_subs_hash}->{$key}->(@args);
+    return $self->{dynamic_subs_hash}->{$key}->(@args);
 }
 
 # Legacy, left in for pre-0.8 obj files
@@ -139,6 +168,8 @@ sub persistent { 0 }
 #
 sub is_subcomp { 0 }
 
+sub is_method { 0 }
+
 #
 # Only true in FileBased subclass.
 #
@@ -176,6 +207,11 @@ sub methods {
     }
 }
 
+#
+# Get all attributes
+#
+sub attributes { $_[0]->{attr} }
+
 #
 # Get attribute by name
 #
@@ -194,6 +230,8 @@ sub attr_if_exists {
     my $value;
     if ($self->_locate_inherited('attr',$name,\$value)) {
 	return $value;
+    } else {
+	return undef;
     }
 }
 
@@ -282,7 +320,9 @@ sub parent {
 
     my $comp;
     if ($self->inherit_path) {
-	$comp = $interp->load($self->inherit_path);
+	$comp = $interp->load($self->inherit_path)
+	    or error(sprintf("cannot find inherit path '%s' for component '%s'",
+			     $self->inherit_path, $self->title));
     } elsif ($self->inherit_start_path) {
 	$comp = $interp->find_comp_upwards($self->inherit_start_path, $interp->autohandler_name);
     }
@@ -290,6 +330,22 @@ sub parent {
     return $comp;
 }
 
+sub interp {
+    my $self = shift;
+
+    if (@_) {
+        validate_pos( @_, { isa => 'HTML::Mason::Interp' } );
+
+        $self->{interp} = $_[0];
+
+        Scalar::Util::weaken( $self->{interp} ) if can_weaken;
+    } elsif ( ! defined $self->{interp} ) {
+        die "The Interp object that this object contains has gone out of scope.\n";
+    }
+
+    return $self->{interp};
+}
+
 #
 # Accessors for various files associated with component
 #
@@ -298,6 +354,12 @@ sub object_file {
     return $self->interp->object_file($self);
 }
 
+# For backwards compatibility with 1.0x
+sub create_time {
+    my $self = shift;
+    return $self->load_time(@_);
+}
+
 1;
 
 __END__
@@ -349,14 +411,14 @@ directly with a component object.
 =head2 CREATING AND ACCESSING COMPONENTS
 
 Common ways to get handles on existing component objects include the
-L<Request-E<gt>current_comp|HTML::Mason::Request/current_comp>,
-L<Request-E<gt>callers|HTML::Mason::Request/callers>, and
-L<Request-E<gt>fetch_comp|HTML::Mason::Request/fetch_comp> methods.
+L<Request-E<gt>current_comp|HTML::Mason::Request/item_current_comp>,
+L<Request-E<gt>callers|HTML::Mason::Request/item_callers>, and
+L<Request-E<gt>fetch_comp|HTML::Mason::Request/item_fetch_comp> methods.
 
 There is no published C<new> method, because creating a component
-requires an Interpreter. Use the L<Interp's
-make_component|HTML::Mason::Interp/make_component> method to create a
-new component dynamically.
+requires an Interpreter. Use the
+L<make_component|HTML::Mason::Interp/item_make_component> method to
+create a new component dynamically.
 
 Similarly, there is no C<execute> or C<call> method, because calling a
 component requires a request. All of the interfaces for calling a
@@ -384,10 +446,21 @@ attribute does not exist.
 Returns true if the specified attribute exists in this component or
 one of its parents, undef otherwise.
 
-=item load_time
+=item attributes
 
-Returns the time (in Perl time() format) when this component object
-was created.
+Returns a hashref containing the attributes defined in this component,
+with the attribute names as keys.  This does not return attributes
+inherited from parent components.
+
+=item call_method (name, args...)
+
+Looks for the specified user-defined method in this component and its
+parents, calling the first one found. Dies with an error if not found.
+Methods are declared in the C<E<lt>%methodE<gt>> section.
+
+=item create_time
+
+A synonym for L<load_time|HTML::Mason::Component/item_load_time> (deprecated).
 
 =item declared_args
 
@@ -431,24 +504,40 @@ Unlike attributes, flags values do not get inherited from parent components.
 
 =item is_subcomp
 
-Returns true if this is a subcomponent of another component.
+Returns true if this is a subcomponent of another component.  For
+historical reasons, this returns true for both methods and
+subcomponents.
+
+=item is_method
+
+Returns true if this is a method.
 
 =item is_file_based
 
 Returns true if this component was loaded from a source or object
 file.
 
-=item call_method (name, args...)
+=for html <a name="item_load_time"></a>
 
-Looks for the specified user-defined method in this component and its
-parents, calling the first one found. Dies with an error if not found.
-Methods are declared in the C<E<lt>%methodE<gt>> section.
+=item load_time
+
+Returns the time (in Perl time() format) when this component object
+was created.
 
 =item method_exists (name)
 
 Returns true if the specified user-defined method exists in this
 component or one of its parents, undef otherwise.
 
+=item methods
+
+This method works exactly like the
+L<subcomps|HTML::Mason::Component/item_subcomps> method, but it
+returns methods, not subcomponents.  This does not return methods
+inherited from parent components.
+
+Methods are declared in C<E<lt>%methodE<gt>> sections.
+
 =item name
 
 Returns a short name of the component.  For file-based components this
@@ -476,10 +565,11 @@ Returns the entire path of this component, relative to the component root.
 
 =item scall_method (name, args...)
 
-Like L<Component-E<gt>call_method (name, args...)|call_method (name,
-args...)>, but returns the method output as a string instead of
-printing it. (Think sprintf versus printf.) The method's return value,
-if any, is discarded.
+Like L<item_call_method|call_method>, but returns the method output as
+a string instead of printing it. (Think sprintf versus printf.) The
+method's return value, if any, is discarded.
+
+=for html <a name="item_subcomps"></a>
 
 =item subcomps
 
@@ -494,13 +584,6 @@ or undef if no such subcomponent exists. e.g.
 
 Subcomponents are declared in C<E<lt>%defE<gt>> sections.
 
-=item methods
-
-This method works exactly like the subcomps method, but it returns
-methods, not subcomponents.
-
-Methods are declared in C<E<lt>%methodE<gt>> sections.
-
 =item title
 
 Returns a printable string denoting this component.  It is intended to
