@@ -4,7 +4,7 @@ use strict;
 
 use vars qw($VERSION);
 
-$VERSION = sprintf '%2d.%02d', q$Revision: 1.39 $ =~ /(\d+)\.(\d+)/;
+$VERSION = 1.43;
 
 my %e;
 
@@ -12,54 +12,58 @@ BEGIN
 {
     %e = ( 'HTML::Mason::Exception' =>
 	   { description => 'generic base class for all Mason exceptions',
-	     abbr => 'error'},
+	     alias => 'error'},
 
 	   'HTML::Mason::Exception::Abort' =>
 	   { isa => 'HTML::Mason::Exception',
-	     abbr => 'abort_error',
 	     fields => [qw(aborted_value)],
 	     description => 'a component called $m->abort' },
 
+	   'HTML::Mason::Exception::Decline' =>
+	   { isa => 'HTML::Mason::Exception',
+	     fields => [qw(declined_value)],
+	     description => 'a component called $m->decline' },
+
 	   'HTML::Mason::Exception::Compiler' =>
 	   { isa => 'HTML::Mason::Exception',
-	     abbr => 'compiler_error',
+	     alias => 'compiler_error',
 	     description => 'error thrown from the compiler' },
 
 	   'HTML::Mason::Exception::Compilation' =>
 	   { isa => 'HTML::Mason::Exception',
-	     abbr => 'compilation_error',
+	     alias => 'compilation_error',
 	     fields => [qw(filename)],
 	     description => "error thrown in eval of the code for a component" },
 
 	   'HTML::Mason::Exception::Compilation::IncompatibleCompiler' =>
-	   { isa => 'HTML::Mason::Exception',
-	     abbr => 'wrong_compiler_error',
+	   { isa => 'HTML::Mason::Exception::Compilation',
+	     alias => 'wrong_compiler_error',
 	     description => "a component was compiled by a compiler/lexer with incompatible options.  recompilation is needed" },
 
 	   'HTML::Mason::Exception::Params' =>
 	   { isa => 'HTML::Mason::Exception',
-	     abbr => 'param_error',
+	     alias => 'param_error',
 	     description => 'invalid parameters were given to a method/function' },
 
 	   'HTML::Mason::Exception::Syntax' =>
 	   { isa => 'HTML::Mason::Exception',
-	     abbr => 'syntax_error',
+	     alias => 'syntax_error',
 	     fields => [qw(source_line comp_name line_number)],
 	     description => 'invalid syntax was found in a component' },
 
 	   'HTML::Mason::Exception::System' =>
 	   { isa => 'HTML::Mason::Exception',
-	     abbr => 'system_error',
+	     alias => 'system_error',
 	     description => 'a system call of some sort failed' },
 
 	   'HTML::Mason::Exception::TopLevelNotFound' =>
 	   { isa => 'HTML::Mason::Exception',
-	     abbr => 'top_level_not_found_error',
+	     alias => 'top_level_not_found_error',
 	     description => 'the top level component could not be found' },
 
 	   'HTML::Mason::Exception::VirtualMethod' =>
 	   { isa => 'HTML::Mason::Exception',
-	     abbr => 'virtual_error',
+	     alias => 'virtual_error',
 	     description => 'a virtual method was not overridden' },
 
 	 );
@@ -69,7 +73,8 @@ use Exception::Class (%e);
 
 HTML::Mason::Exception->Trace(1);
 
-my %abbrs = map { $e{$_}{abbr} => $_ } grep {exists $e{$_}{abbr}} keys %e;
+# To avoid circular reference between exception and request.
+HTML::Mason::Exception->NoRefs(1);
 
 # The import() method allows this:
 #  use HTML::Mason::Exceptions(abbr => ['error1', 'error2', ...]);
@@ -85,14 +90,15 @@ sub import
     {
 	foreach my $name (@{$args{abbr}})
 	{
-	    die "Unknown exception abbreviation '$name'" unless exists $abbrs{$name};
 	    no strict 'refs';
-	    *{"${caller}::$name"} = sub { $abbrs{$name}->throw( error => shift ) };
+	    die "Unknown exception abbreviation '$name'" unless defined &{$name};
+	    *{"${caller}::$name"} = \&{$name};
 	}
     }
     {
 	no strict 'refs';
 	*{"${caller}::isa_mason_exception"} = \&isa_mason_exception;
+	*{"${caller}::rethrow_exception"} = \&rethrow_exception;
     }
 }
 
@@ -101,14 +107,25 @@ sub isa_mason_exception
     my ($err, $name) = @_;
     return unless defined $err;
 
-    if ($name) {
-	my $class = "HTML::Mason::Exception::$name";
-	no strict 'refs';
-	die "no such exception class $class" unless defined(${"${class}::VERSION"});
-	return UNIVERSAL::isa($err, "HTML::Mason::Exception::$name");
-    } else {
-	return UNIVERSAL::isa($err, "HTML::Mason::Exception");
+    $name = $name ? "HTML::Mason::Exception::$name" : "HTML::Mason::Exception";
+    no strict 'refs';
+    die "no such exception class $name" unless defined(${"${name}::VERSION"});
+
+    return UNIVERSAL::isa($err, $name);
+}
+
+sub rethrow_exception
+{
+    my ($err) = @_;
+    return unless $err;
+
+    if ( UNIVERSAL::can($err, 'rethrow') ) {
+	$err->rethrow;
     }
+    elsif ( ref $err ) {
+        die $err;
+    }
+    HTML::Mason::Exception->throw(error => $err);
 }
 
 package HTML::Mason::Exception;
@@ -130,7 +147,8 @@ sub new
 # exceptions can get stringified more than once.
 sub throw
 {
-    my ($class, %params) = @_;
+    my $class = shift;
+    my %params = @_ == 1 ? ( error => $_[0] ) : @_;
 
     if (HTML::Mason::Exceptions::isa_mason_exception($params{error})) {
 	$params{error} = $params{error}->error;
@@ -151,8 +169,10 @@ sub filtered_frames
 	qw[
 	   (eval)
 	   Exception::Class::Base::throw
+	   Exception::Class::__ANON__
 	   HTML::Mason::Commands::__ANON__
 	   HTML::Mason::Component::run
+	   HTML::Mason::Exception::throw
 	   HTML::Mason::Exceptions::__ANON__
 	   HTML::Mason::Request::_run_comp
 	   ];
@@ -245,6 +265,14 @@ sub get_file_context
     return @context;
 }
 
+# basically the same as as_string in Exception::Class::Base
+sub raw_text
+{
+    my ($self) = @_;
+
+    return $self->full_message . "\n\n" . $self->trace->as_string;
+}
+
 sub as_string
 {
     my ($self) = @_;
@@ -282,44 +310,114 @@ sub as_text
 sub as_html
 {
     my ($self) = @_;
-    my $info = $self->analyze_error;
 
     my $out;
-    my $msg = HTML::Mason::Tools::html_escape($self->full_message);
-    $msg =~ s/\n/<br>/g;
-    $out .= qq[<html><body>\n];
-    $out .= qq[<p align="center"><font face="Verdana, Arial, Helvetica, sans-serif"><b>System error</b></font></p>\n];
-    $out .= qq[<table border="0" cellspacing="0" cellpadding="1">\n];
-    $out .= qq[<tr>];
-    $out .= qq[<td nowrap align="left" valign="top"><b>error:</b>&nbsp;</td>\n];
-    $out .= sprintf(qq[<td align="left" valign="top" nowrap>%s</td>\n], $msg);
-    $out .= qq[</tr>\n];
-    $out .= qq[<tr>\n];
-    $out .= qq[<td nowrap align="left" valign="top"><b>context:</b>&nbsp;</td>\n];
-    $out .= qq[<td align="left" valign="top" nowrap><table border="0" cellpadding="0" cellspacing="0">\n];
-    foreach my $entry (@{$info->{context}}) {
-	my ($line_num, $line, $highlight) = @$entry;
-	$line = HTML::Mason::Tools::html_escape($line);
-	$line =~ s/ /&nbsp;/g if defined $line;
-	$out .= qq[<tr>\n];
-	$out .= sprintf(qq[<td nowrap align="left" valign="top"><b>%s</b>&nbsp;</td>\n], $line_num);
-	$out .= sprintf(qq[<td align="left" valign="top" nowrap>%s%s%s</td>\n],
-			($highlight ? "<font color=red>" : ""),
-			defined $line ? $line : '',
-			($highlight ? "</font>" : ""));
-	$out .= qq[</tr>\n];
-    }
-    $out .= qq[</table></td></tr>\n];
-    $out .= qq[<tr>\n];
-    $out .= qq[<td align="left" valign="top" nowrap><b>code stack:</b>&nbsp;</td>\n];
-    $out .= qq[<td align="left" valign="top" nowrap><table border="0" cellpadding="0" cellspacing="0">\n];
-    foreach my $frame (@{$info->{frames}}) {
-	$out .= sprintf(qq[%s:%d\n<br>], HTML::Mason::Tools::html_escape($frame->filename), HTML::Mason::Tools::html_escape($frame->line));
-    }
-    $out .= qq[</td>\n];
-    $out .= qq[</tr>\n];
-    $out =~ s/(<td [^\>]+>)/$1<font face="Verdana, Arial, Helvetica, sans-serif" size="-2">/g;
-    $out =~ s/<\/td>/<\/font><\/td>/g;
+    my $interp = HTML::Mason::Interp->new(out_method => \$out);
+
+    # Can't use |h escape in here because if we fail to load
+    # HTML::Entities we end up in an endless loop.
+    my $comp = $interp->make_component(comp_source => <<'EOF');
+
+<%args>
+ $msg
+ $info
+ $error
+</%args>
+<%filter>
+ s/(<td [^\>]+>)/$1<font face="Verdana, Arial, Helvetica, sans-serif" size="-2">/g;
+ s/<\/td>/<\/font><\/td>/g;
+</%filter>
+
+% HTML::Mason::Escapes::basic_html_escape(\$msg);
+% $msg =~ s/\n/<br>/g;
+
+<html><body>
+
+<p align="center"><font face="Verdana, Arial, Helvetica, sans-serif"><b>System error</b></font></p>
+<table border="0" cellspacing="0" cellpadding="1">
+ <tr>
+  <td nowrap="nowrap" align="left" valign="top"><b>error:</b>&nbsp;</td>
+  <td align="left" valign="top"><% $msg %></td>
+ </tr>
+ <tr>
+  <td nowrap="nowrap" align="left" valign="top"><b>context:</b>&nbsp;</td>
+  <td align="left" valign="top" nowrap="nowrap">
+   <table border="0" cellpadding="0" cellspacing="0">
+
+%   foreach my $entry (@{$info->{context}}) {
+%	my ($line_num, $line, $highlight) = @$entry;
+%	$line = '' unless defined $line;
+%       HTML::Mason::Escapes::basic_html_escape(\$line);
+    <tr>
+     <td nowrap="nowrap" align="left" valign="top"><b><% $line_num %></b>&nbsp;</td>
+     <td align="left" valign="top" nowrap="nowrap"><% $highlight ? "<font color=red>" : "" %><% $line %><% $highlight ? "</font>" : "" %></td>
+    </tr>
+
+%    }
+
+   </table>
+  </td>
+ </tr>
+ <tr>
+  <td align="left" valign="top" nowrap="nowrap"><b>code stack:</b>&nbsp;</td>
+  <td align="left" valign="top" nowrap="nowrap">
+%    foreach my $frame (@{$info->{frames}}) {
+%        my $f = $frame->filename; HTML::Mason::Escapes::basic_html_escape(\$f);
+%        my $l = $frame->line; HTML::Mason::Escapes::basic_html_escape(\$l);
+	<% $f %>:<% $l %><br>
+%    }
+  </td>
+ </tr>
+</table>
+
+<a href="#raw">raw error</a><br>
+
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+<br>
+
+% my $raw = $error->raw_text;
+% HTML::Mason::Escapes::basic_html_escape(\$raw);
+% $raw =~ s/\t//g;
+
+<a name="raw"></a>
+
+<pre><% $raw %></pre>
+
+</body></html>
+EOF
+
+    $interp->exec($comp,
+                  msg => $self->full_message,
+                  info => $self->analyze_error,
+                  error => $self);
 
     return $out;
 }
@@ -350,12 +448,18 @@ __END__
 
 HTML::Mason::Exceptions - Exception objects thrown by Mason
 
-=head1 DESCRIPTION
+=head1 SYNOPSIS
 
-  use HTML::Mason::Exceptions qw(system_error);
+  use HTML::Mason::Exceptions ( abbr => [ qw(system_error) ] );
 
   open FH, 'foo' or system_error "cannot open foo: $!";
 
+=head1 DESCRIPTION
+
+This module creates the hierarchy of exception objects used by Mason,
+and provides some extra methods for them beyond those provided by
+C<Exception::Class>
+
 =head1 IMPORT
 
 When this module is imported, it is possible to specify a list of
@@ -384,7 +488,11 @@ The C<< $m->abort >> method was called.
 
 Exceptions in this class contain the field C<aborted_value>.
 
-Abbreviated as C<abort_error>.
+=item HTML::Mason::Exception::Decline
+
+The C<< $m->decline >> method was called.
+
+Exceptions in this class contain the field C<declined_value>.
 
 =item HTML::Mason::Exception::Compilation
 
