@@ -1,4 +1,4 @@
-# Copyright (c) 1998-2002 by Jonathan Swartz. All rights reserved.
+# Copyright (c) 1998-2005 by Jonathan Swartz. All rights reserved.
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 
@@ -8,11 +8,13 @@ use strict;
 
 use HTML::Mason::Exceptions( abbr => [qw(param_error syntax_error error)] );
 
+use HTML::Mason::Tools qw( taint_is_on );
+
 use Params::Validate qw(:all);
 Params::Validate::validation_options( on_fail => sub { param_error join '', @_ } );
 
-use HTML::Mason::Container;
-use base qw(HTML::Mason::Container);
+use Class::Container;
+use base qw(Class::Container);
 
 # This is a block name and what method should be called to lex its
 # contents if it is encountered.  'def' & 'method' blocks are special
@@ -59,11 +61,16 @@ sub lex
 {
     my $self = shift;
     my %p = validate(@_,
-		     {comp_source => SCALAR,
+		     {comp_source => SCALAR|SCALARREF,
 		      name => SCALAR,
 		      compiler => {isa => 'HTML::Mason::Compiler'}}
 		    );
 
+    # Note - we could improve memory usage here if we didn't make a
+    # copy of the scalarref, but that will take some more work to get
+    # it working
+    $p{comp_source} = ${$p{comp_source}} if ref $p{comp_source};
+
     # Holds information about the current lex.  Make it local() so
     # we're fully re-entrant.
     local $self->{current} = \%p;
@@ -75,7 +82,6 @@ sub lex
     # Initialize lexer state
     $current->{lines} = 1;
     $current->{in_def} = $current->{in_method} = 0;
-    $current->{pos} = undef;
 
     # This will be overridden if entering a def or method section.
     $current->{ending} = qr/\G\z/;
@@ -83,39 +89,23 @@ sub lex
     # We need to untaint the component or else the regexes will fail
     # to a Perl bug.  The delete is important because we need to
     # create an entirely new scalar, not just modify the existing one.
-    ($current->{comp_source}) = delete($current->{comp_source}) =~ /(.*)/s;
+    ($current->{comp_source}) = (delete $current->{comp_source}) =~ /(.*)/s
+        if taint_is_on;
 
     eval
     {
 	$current->{compiler}->start_component;
 	$self->start;
     };
-    # Call this out here because it may be needed to break circular
-    # refs inside the compiler
-    $current->{compiler}->end_component;
-
-    if ($@)
+    my $err = $@;
+    # Always call end_component, but throw the first error
+    eval
     {
-	$@->rethrow if UNIVERSAL::can( $@, 'rethrow' );
-	error $@;
-    }
-}
+        $current->{compiler}->end_component;
+    };
+    $err ||= $@;
 
-sub object_id
-{
-    my $self = shift;
-
-    my @vals;
-    foreach my $k ( sort keys %{ $self->validation_spec } )
-    {
-	push @vals, $k;
-	push @vals, ( UNIVERSAL::isa( $self->{$k}, 'HASH' )  ? map { $_ => $self->{$k}{$_} } sort keys %{ $self->{$k} } :
-		      UNIVERSAL::isa( $self->{$k}, 'ARRAY' ) ? sort @{ $self->{$k} } :
-		      $self->{$k} );
-    }
-
-    # unpack('%32C*', $x) computes the 32-bit checksum of $x
-    return unpack('%32C*', join "\0", class => ref($self), @vals);
+    rethrow_exception $err;
 }
 
 sub start
@@ -123,9 +113,7 @@ sub start
     my $self = shift;
 
     my $end;
-    while ( defined $self->{current}{pos} ?
-	    $self->{current}{pos} < length $self->{current}{comp_source} :
-	    1 )
+    while (1)
     {
 	last if $end = $self->match_end;
 
@@ -152,6 +140,8 @@ sub start
 	    $self->throw_syntax_error("Missing closing </%$type> tag");
 	}
 
+	last if $self->{current}{comp_source} =~ /\G\z/;
+
 	# We should never get here - if we do, we're in an infinite loop.
 	$self->throw_syntax_error("Infinite parsing loop encountered - Lexer bug?");
     }
@@ -179,7 +169,7 @@ sub match_block
 	$self->{current}{compiler}->start_block( block_type => $type );
 
 	my $method = $self->block_body_method($type);
-	$self->$method( block_type => $type );
+	$self->$method( {block_type => $type} );
 
 	return 1;
     }
@@ -187,45 +177,43 @@ sub match_block
 
 sub generic_block
 {
-    my $self = shift;
-    my %p = @_;
+    my ($self, $method, $p) = @_;
 
-    my ($block, $nl) = $self->match_block_end( block_type => $p{block_type},
-					       allow_text => 1 );
+    $p->{allow_text} = 1;
+    my ($block, $nl) = $self->match_block_end( $p );
 
-    my $method = $p{method};
-    $self->{current}{compiler}->$method( block_type => $p{block_type},
+    $self->{current}{compiler}->$method( block_type => $p->{block_type},
 					 block => $block );
 
-    $self->{current}{lines} += $block =~ tr/\n/\n/;
+    $self->{current}{lines} += $block =~ tr/\n//;
     $self->{current}{lines}++ if $nl;
 
-    $self->{current}{compiler}->end_block( block_type => $p{block_type} );
+    $self->{current}{compiler}->end_block( block_type => $p->{block_type} );
 }
 
 sub text_block
 {
     my $self = shift;
-    $self->generic_block(@_, method => 'text_block');
+    $self->generic_block('text_block', @_);
 }
 
 sub raw_block
 {
     my $self = shift;
-    $self->generic_block(@_, method => 'raw_block');
+    $self->generic_block('raw_block', @_);
 }
 
 sub doc_block
 {
     my $self = shift;
-    $self->generic_block(@_, method => 'doc_block');
+    $self->generic_block('doc_block', @_);
 }
 
 sub variable_list_block
 {
-    my ($self, %p) = @_;
+    my ($self, $p) = @_;
 
-    my $ending = qr/ \n | <\/%\Q$p{block_type}\E> /x;
+    my $ending = qr/ \n | <\/%\Q$p->{block_type}\E> /ix;
     while ( $self->{current}{comp_source} =~ m,
                        \G               # last pos matched
                        (?:
@@ -257,47 +245,53 @@ sub variable_list_block
                         |
                         [ \t]*          # just space
                        )
-                       (?:\n |          # newline or
-                          (?= <\/%\Q$p{block_type}\E> ) )   # end of block (don't consume it)
-                      ,xgc
+                       (\n |          # newline or
+                          (?= <\/%\Q$p->{block_type}\E> ) )   # end of block (don't consume it)
+                      ,ixgc
 	  )
     {
 	if ( defined $1 && defined $2 && length $1 && length $2 )
 	{
-	    $self->{current}{compiler}->variable_declaration( block_type => $p{block_type},
+	    $self->{current}{compiler}->variable_declaration( block_type => $p->{block_type},
 							      type => $1,
 							      name => $2,
 							      default => $3,
 							    );
 	}
 
-	$self->{current}{lines}++;
+	$self->{current}{lines}++ if $4;
     }
 
-    my $nl = $self->match_block_end( block_type => $p{block_type},
-				     allow_text => 0 );
+    $p->{allow_text} = 0;
+    my $nl = $self->match_block_end( $p );
     $self->{current}{lines}++ if $nl;
 
-    $self->{current}{compiler}->end_block( block_type => $p{block_type} );
+    $self->{current}{compiler}->end_block( block_type => $p->{block_type} );
 }
 
 sub key_val_block
 {
-    my ($self, %p) = @_;
+    my ($self, $p) = @_;
 
     my $ending = qr, (?: \n |           # newline or
-                         (?= </%\Q$p{block_type}\E> ) )   # end of block (don't consume it)
-                   ,x;
+                         (?= </%\Q$p->{block_type}\E> ) )   # end of block (don't consume it)
+                   ,ix;
 
     while ( $self->{current}{comp_source} =~ /
                       \G
                       [ \t]*
                       ([\w_]+)          # identifier
                       [ \t]*=>[ \t]*    # separator
-                      (\S[^\n]*?)        # value ( must start with a non-space char)
+                      (\S[^\n]*?)       # value ( must start with a non-space char)
                       $ending
                       |
-                      \G\n
+                      \G\n              # a plain empty line
+                      |
+                      \G
+                      [ \t]*            # an optional comment
+                      \#
+                      [^\n]*
+                      $ending
                       |
                       \G[ \t]+?
                       $ending
@@ -305,7 +299,7 @@ sub key_val_block
     {
 	if ( defined $1 && defined $2 && length $1 && length $2 )
 	{
-	    $self->{current}{compiler}->key_value_pair( block_type => $p{block_type},
+	    $self->{current}{compiler}->key_value_pair( block_type => $p->{block_type},
 							key => $1,
 							value => $2
 						      );
@@ -314,36 +308,40 @@ sub key_val_block
 	$self->{current}{lines}++;
     }
 
-    my $nl = $self->match_block_end( block_type => $p{block_type},
-				     allow_text => 0 );
+    $p->{allow_text} = 0;
+    my $nl = $self->match_block_end( $p );
     $self->{current}{lines}++ if $nl;
 
-    $self->{current}{compiler}->end_block( block_type => $p{block_type} );
+    $self->{current}{compiler}->end_block( block_type => $p->{block_type} );
 }
 
 sub match_block_end
 {
-    my ($self, %p) = @_;
+    my ($self, $p) = @_;
 
-    my $re = $p{allow_text} ? qr,\G(.*?)</%\Q$p{block_type}\E>(\n?),is
-                            : qr,\G()\s*</%\Q$p{block_type}\E>(\n?),is;
+    my $re = $p->{allow_text} ? qr,\G(.*?)</%\Q$p->{block_type}\E>(\n?),is
+                              : qr,\G\s*</%\Q$p->{block_type}\E>(\n?),is;
     if ( $self->{current}{comp_source} =~ /$re/gc )
     {
-	return $p{allow_text} ? ($1, $2) : $2;
+	return $p->{allow_text} ? ($1, $2) : $1;
     }
     else
     {
-	$self->throw_syntax_error("Invalid <%$p{block_type}> section line");
+	$self->throw_syntax_error("Invalid <%$p->{block_type}> section line");
     }
 }
 
 sub match_named_block
 {
-    my ($self, %p) = @_;
+    my ($self, $p) = @_;
 
-    if ( $self->{current}{comp_source} =~ /\G<%(def|method)\s+([^\n]+?)>/igcs )
+    if ( $self->{current}{comp_source} =~ /\G<%(def|method)(?:\s+([^\n]+?))?\s*>/igcs )
     {
-	my ($type, $name) = ($1, $2);
+	my ($type, $name) = (lc $1, $2);
+
+	$self->throw_syntax_error("$type block without a name")
+	    unless defined $name && length $name;
+
 	$self->{current}{compiler}->start_named_block( block_type => $type,
 						       name => $name );
 
@@ -361,28 +359,61 @@ sub match_named_block
     }
 }
 
+# starts with an alpha character or underscore, followed by any word
+# character
+my $flag;
+if ( $] >= 5.006 )
+{
+    # Unicode-friendly
+
+    # Use eval because 5.00503 complains even if it doesn't execute
+    # it.
+    eval '$flag = qr/[[:alpha:]_]\w*/;';
+}
+else
+{
+    # Like [a-zA-Z_] but respects locales
+    $flag = qr/[^\W\d]\w*/x;
+}
+
+sub escape_flag_regex { $flag }
+
 sub match_substitute
 {
+    # This routine relies on there *not* to be an opening <%foo> tag
+    # present, so match_block() must happen first.
+    
     my $self = shift;
 
-    if ( $self->{current}{comp_source} =~ /\G<%/gcs )
+    return 0 unless $self->{current}{comp_source} =~ /\G<%/gcs;
+
+    if ( $self->{current}{comp_source} =~
+	 m{
+	   \G
+	   (.+?)                # Substitution body ($1)
+	   (
+	    \s*
+	    (?<!\|)             # Not preceded by a '|'
+	    \|                  # A '|'
+	    \s*
+	    (                   # (Start $3)
+	     $flag              # A flag
+	     (?:\s*,\s*$flag)*  # More flags, with comma separators
+	    )
+	    \s*
+	   )?
+	   %>                   # Closing tag
+	  }xcigs )
     {
-	if ( $self->{current}{comp_source} =~ /\G(.+?)(\s*\|\s*([a-z]+)?\s*)?%>/igcs )
-	{
-	    my ($sub, $escape) = ($1, $3);
-	    $self->{current}{compiler}->substitution( substitution => $sub,
-						      escape => $escape );
+	$self->{current}{lines} += tr/\n// foreach grep defined, ($1, $2);
 
-	    # Add it in just to count lines
-	    $sub .= $2 if $2;
-	    $self->{current}{lines} += $sub =~ tr/\n/\n/;
-
-	    return 1;
-	}
-	else
-	{
-	    $self->throw_syntax_error("'<%' without matching '%>'");
-	}
+	$self->{current}{compiler}->substitution( substitution => $1,
+						  escape => $3 );
+	return 1;
+    }
+    else
+    {
+	$self->throw_syntax_error("'<%' without matching '%>'");
     }
 }
 
@@ -396,7 +427,7 @@ sub match_comp_call
 	{
 	    my $call = $1;
 	    $self->{current}{compiler}->component_call( call => $call );
-	    $self->{current}{lines} += $call =~ tr/\n/\n/;
+	    $self->{current}{lines} += $call =~ tr/\n//;
 
 	    return 1;
 	}
@@ -418,7 +449,7 @@ sub match_comp_content_call
 	{
 	    my $call = $1;
 	    $self->{current}{compiler}->component_content_call( call => $call );
-	    $self->{current}{lines} += $call =~ tr/\n/\n/;
+	    $self->{current}{lines} += $call =~ tr/\n//;
 
 	    return 1;
 	}
@@ -433,9 +464,11 @@ sub match_comp_content_call_end
 {
     my $self = shift;
 
-    if ( $self->{current}{comp_source} =~ m,\G</&>,gc )
+    if ( $self->{current}{comp_source} =~ m,\G</&(.*?)>,gc )
     {
-        $self->{current}{compiler}->component_content_call_end;
+	my $call = $1 || '';
+        $self->{current}{compiler}->component_content_call_end( call_end => $call );
+	$self->{current}{lines} += $call =~ tr/\n//;
 
         return 1;
     }
@@ -445,7 +478,7 @@ sub match_perl_line
 {
     my $self = shift;
 
-    if ( $self->{current}{comp_source} =~ /\G%([^\n]*)(?:\n|\z)/gcs )
+    if ( $self->{current}{comp_source} =~ /\G(?<=^)%([^\n]*)(?:\n|\z)/gcm )
     {
 	$self->{current}{compiler}->perl_line( line => $1 );
 	$self->{current}{lines}++;
@@ -457,69 +490,39 @@ sub match_perl_line
 sub match_text
 {
     my $self = shift;
+    my $c = $self->{current};
 
-    #
-    # This regex is a bit weird, to say the least!
-    #
-    # Basically, we need to first check for "text" at the current
-    # position that we don't pass through as text.  This is either an
-    # escaped newline or an EOF.
-    #
-    # If we find one of those, we have matched some "text" but nothing
-    # that we pass through to the compiler.
-    #
-    # Otherwise, we try to match _at least_ one character of text
-    # following by some non-text production, which may be some sort of
-    # Mason syntax or an escaped newline or an EOF.
-    #
-    # It is important that we match at least one character or we can
-    # fail incorrectly on something like this:
-    #
-    #   <%foo>
-    #
-    # It doesn't match any of the previous productions but if found at
-    # the beginning of a component it will not match as text unless we
-    # can consume the first character, "<", before trying to match any
-    # more Mason syntax.
-    #
-    # Otherwise we'd match the empty string at the beginning, followed
-    # by "<%".
-    #
-    # - Dave - 1/6/2002
-    #
-    if ( $self->{current}{comp_source} =~ m,
-                    \G
-                    (?:
-                     (\\\n)      # an escaped newline  - throw away
-                     |
-                     \z          # or EOF.
-                     |
-                     (?:
-                      (.+?)       # anything
-	              (           # followed by
-                       (?<=\n)(?=%) # an eval line - consume the \n
-                       |
-                       (?=</?%)   # a substitution or tag start or end  - don't consume
-                       |
-                       (?=</?&)   # a comp call start or end  - don't consume
-                       |
-                       \\\n       # an escaped newline  - throw away
-                       |
-                       \z         # or EOF.
-                      )
-                     )
-                    )
-                   ,gcsx
-       )
+    # Most of these terminator patterns actually belong to the next
+    # lexeme in the source, so we use a lookahead if we don't want to
+    # consume them.  We use a lookbehind when we want to consume
+    # something in the matched text, like the newline before a '%'.
+    if ( $c->{comp_source} =~ m{
+				\G
+				(.*?)         # anything, followed by:
+				(
+				 (?<=\n)(?=%) # an eval line - consume the \n
+				 |
+				 (?=</?[%&])  # a substitution or block or call start or end
+                                              # - don't consume
+				 |
+				 \\\n         # an escaped newline  - throw away
+				 |
+				 \z           # end of string
+				)
+			       }xcgs )
     {
-	my $consumed = join '', grep { defined } $1, $2, $3;
-	return 0 unless length $consumed;
+	# Note: to save memory, it might be preferable to break very
+	# large $1 strings into several pieces and pass the pieces to
+	# compiler->text().  In my testing, this was quite a bit
+	# slower, though.  -Ken 2002-09-19
+	$c->{compiler}->text( text => $1 ) if length $1;
+        # Not checking definedness seems to cause extra lines to be
+        # counted with Perl 5.00503.  I'm not sure why - dave
+	$c->{lines} += tr/\n// foreach grep defined, ($1, $2);
 
-	$self->{current}{compiler}->text( text => $2 ) if defined $2 && length $2;
-
-	$self->{current}{lines} += $consumed =~ tr/\n/\n/;
 	return 1;
     }
+    
     return 0;
 }
 
@@ -531,14 +534,10 @@ sub match_end
     # also include the needed \G marker
     if ( $self->{current}{comp_source} =~ /($self->{current}{ending})/gcs )
     {
-	my $text = $1;
-	if (defined $text)
-	{
-	    $self->{current}{lines} += $text =~ tr/\n/\n/;
-	}
-
-	return $1 || 1;
+	$self->{current}{lines} += $1 =~ tr/\n//;
+	return defined $1 && length $1 ? $1 : 1;
     }
+    return 0;
 }
 
 # goes from current pos, skips a newline if its the next character,
@@ -662,4 +661,8 @@ in Mason, you must implement the methods listed above.  If you plan to
 use a custom Compiler class that you're writing, you can do whatever
 you want.
 
+We recommend that any parameters you add to Lexer be read-only,
+because the compiler object_id is only computed once on creation
+and would not reflect any changes to Lexer parameters.
+
 =cut
