@@ -1,4 +1,4 @@
-# Copyright (c) 1998-2002 by Jonathan Swartz. All rights reserved.
+# Copyright (c) 1998-2005 by Jonathan Swartz. All rights reserved.
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 
@@ -21,8 +21,8 @@ use vars qw(@ISA @EXPORT_OK);
 
 sub data_cache_namespace
 {
-    my ($path) = @_;
-    return compress_path($path);
+    my ($comp_id) = @_;
+    return compress_path($comp_id);
 }
 
 sub cgi_request_args
@@ -36,16 +36,21 @@ sub cgi_request_args
     # parameter named 'keywords' with a value of () (empty array).
     # This is apparently a feature related to <ISINDEX> queries or
     # something (see the CGI.pm) docs.  It makes my head hurt. - dave
-    my @methods = $method ne 'POST' || ! $ENV{QUERY_STRING} ? ( 'param' ) : ( 'param', 'url_param' );
+    my @methods =
+        $method ne 'POST' || ! $ENV{QUERY_STRING} ? ( 'param' ) : ( 'param', 'url_param' );
+
     foreach my $key ( map { $q->$_() } @methods ) {
 	next if exists $args{$key};
 	my @values = map { $q->$_($key) } @methods;
 	$args{$key} = @values == 1 ? $values[0] : \@values;
     }
 
-    return %args;
+    return wantarray ? %args : \%args;
 }
 
+
+1;
+
 __END__
 
 =head1 NAME
@@ -61,9 +66,25 @@ code you have written with Mason.
 
 =over 4
 
-=item data_cache_namespace
+=item data_cache_namespace ($comp_id)
 
-=item cgi_request_args
+Given a component id, this method returns its default
+C<Cache::Cache> namespace.  This can be useful if you want to access
+the cached data outside of Mason.
+
+With a single component root, the component id is just the component
+path. With multiple component roots, the component id is
+C<key>/C<path>, where C<key> is the key corresponding to the root that
+the component falls under.
+
+=item cgi_request_args ($cgi, $method)
+
+This function expects to receive a C<CGI.pm> object and the request
+method (GET, POST, etc).  Given these two things, it will return a
+hash in list context or a hashref in scalar context.  The hash(ref)
+will contain all the arguments passed via the CGI request.  The keys
+will be argument names and the values will be either scalars or array
+references.
 
 =back
 
